//-----------------------------------------------------------------------------
// Constants
//-----------------------------------------------------------------------------

// UsbI2cIo maximum devices
#define USBI2CIO_MAX_DEVICES            127
#define USBI2CIO_VR_MAX_DATA            1088
#define USBI2CIO_I2C_MAX_DATA           USBI2CIO_VR_MAX_DATA
#define USBI2CIO_IO_MAX_DATA            USBI2CIO_VR_MAX_DATA

// I2C transaction types
typedef enum {
  // standard transaction types
  I2C_TRANS_NOADR = 0x00,         // read or write with no address cycle
  I2C_TRANS_8ADR,                 // read or write with 8 bit address cycle
  I2C_TRANS_16ADR,                // read or write with 16 bit address cycle
  I2C_TRANS_NOADR_NS,             // read or write with no address cycle, stop signaling inhibited
  I2C_TRANS_XICOR,                // read or write with 8 bit instruction cycle, non I2C compliant use of R/W bit

  // customer specific transactions types for accessing rams, only write is mode supported
  I2C_TRANS_8ADR_NONSEQ = 0x30,   // read or write with 8 bit address cycle, non sequential
  I2C_TRANS_16ADR_NONSEQ,         // read or write with 16 bit address cycle, non sequential
  I2C_TRANS_24ADR_NONSEQ          // read or write with 24 bit address cycle, non sequential

} I2C_TRANS_TYPE;

// Properties constants
typedef enum {
  // MAX PROPERTIES (byte 0)
  PROPERTY_WRCOMMAND_RDCOUNT = 0x00,          // when setting, indicates command, when reading indicates table size
  // I2C PROPERTIES (byte 1)
  PROPERTY_I2C_CONFIG,
  // FAST TRANSFER PROPERTIES (byte 2)
  PROPERTY_FAST_XFER_CONFIG,
  // I/O PINS GLOBAL PROPERTIES (byte 3)
  PROPERTY_IO_CONFIG_GLOBAL,
  // I/O PINS PORT CONFIG PROPERTIES (bytes 4-6)
  PROPERTY_IO_CONFIG_PORTA,
  PROPERTY_IO_CONFIG_PORTB,
  PROPERTY_IO_CONFIG_PORTC,
  // I/O PINS PORT OUTPUT PROPERTIES (bytes 7-9)
  PROPERTY_IO_OUTPUT_PORTA,
  PROPERTY_IO_OUTPUT_PORTB,
  PROPERTY_IO_OUTPUT_PORTC,
  // DEBUG PROPERTIES (bytes 10-12)
  PROPERTY_DEBUG_CONFIG_GLOBAL,
  PROPERTY_DEBUG_CONFIG_0,
  PROPERTY_DEBUG_CONFIG_1,
  // USER PROPERTIES (bytes 13-16)
  PROPERTY_USER_0,
  PROPERTY_USER_1,
  PROPERTY_USER_2,
  PROPERTY_USER_3,
  // I/O PINS
  PROPERTY_IO_CONFIG_PORTD,
  PROPERTY_IO_OUTPUT_PORTD,
  // OUTPUT MODE (Pusb-Pull or Open-Drain)
  PROPERTY_IO_OUTPUT_MODE_PORTA,
  PROPERTY_IO_OUTPUT_MODE_PORTB,
  PROPERTY_IO_OUTPUT_MODE_PORTC,
  PROPERTY_IO_OUTPUT_MODE_PORTD,
  // I2C DEFAULT CHANNEL SELECTION AND CLOCK SPEED
  PROPERTY_I2C_DEFAULT_CHANNEL,
  PROPERTY_I2C_CHAN0_CLK_LO,
  PROPERTY_I2C_CHAN0_CLK_HI,
  PROPERTY_I2C_CHAN1_CLK_LO,
  PROPERTY_I2C_CHAN1_CLK_HI,
  PROPERTY_I2C_CHAN2_CLK_LO,
  PROPERTY_I2C_CHAN2_CLK_HI,
  PROPERTY_MAX_PROPERTY_BYTES
} USBI2CIO_PROPERTY_OFFSETS;

typedef enum {
  PROPERTY_CMD_STORE_TABLE_TO_EEPROM = 0x00,
  PROPERTY_CMD_LOAD_TABLE_FROM_EEPROM,
  PROPERTY_CMD_DISABLE_EEPROM_TABLE,
  PROPERTY_CMD_RESET_TO_DEFAULTS
} USBI2CIO_PROPERTY_COMMANDS;


// MAX PROPERTIES (byte 0)
//#define PROP_MAX_PROPERTIES               0x00


// I2C PROPERTIES (byte 1)
//#define PROP_I2C_BYTE                     0x01

  #define PROP_I2C_RETRIES_FIELD          0x07
  #define PROP_I2C_IGNORE_NAK             0x08
  #define PROP_I2C_POLL_EEPROM_ACK        0x10
  #define PROP_I2C_AUTO_REDIRECT_A2_REQS  0x20
  #define PROP_I2C_RESERVED_FIELD         0xC0

// FAST TRANSFER PROPERTIES (byte 2)
//#define PROP_FAST_XFER_BYTE               0x02

  #define PROP_FAST_XFER_RD_FIELD         0x07
  #define PROP_FAST_XFER_RD_ENABLE        0x08
  #define PROP_FAST_XFER_WR_FIELD         0x70
  #define PROP_FAST_XFER_WR_ENABLE        0x80

// I/O PINS GLOBAL PROPERTIES (byte 3)
//#define PROP_IO_GLOBAL_CONFIG_BYTE        0x03
  #define PROP_IO_23BIT_MODE              0x01

// I/O PINS PORT CONFIG PROPERTIES (bytes 4-6)
//#define PROP_IO_PORTA_CONFIG_BYTE         0x04
//#define PROP_IO_PORTB_CONFIG_BYTE         0x05
//#define PROP_IO_PORTC_CONFIG_BYTE         0x06

// I/O PINS PORT OUTPUT PROPERTIES (bytes 7-9)
//#define PROP_IO_PORTA_OUTPUT_BYTE         0x07
//#define PROP_IO_PORTB_OUTPUT_BYTE         0x08
//#define PROP_IO_PORTC_OUTPUT_BYTE         0x09

// DEBUG PROPERTIES (bytes 10-12)
//  PROPERTY_DEBUG_CONFIG_GLOBAL,
  #define PROP_DBG_MAX_DISPLAY_FIELD      0x7F
  #define PROP_DBG_GLOBAL_ENABLE          0x80

//  PROPERTY_DEBUG_CONFIG_0,
//  PROPERTY_DEBUG_CONFIG_1,

// USER PROPERTIES (bytes 10-13)
//#define PROP_USER_0                       0x0A
//#define PROP_USER_1                       0x0B
//#define PROP_USER_2                       0x0C
//#define PROP_USER_3                       0x0D

//#define PROP_MAX_PROP_BYTES               0x0E


//-----------------------------------------------------------------------------
// Structure Definitions
//-----------------------------------------------------------------------------

typedef struct _DEV_INFO {                    // structure for device information
  BYTE byInstance;
  char SerialId[9];
} DEV_INFO, *LPDEV_INFO;

typedef struct _DEV_INFO_ARRAY {              // structure for array of device information structures
  DEV_INFO DevInfo[USBI2CIO_MAX_DEVICES];
} DEV_INFO_ARRAY, *LPDEV_INFO_ARRAY;

#pragma pack(push, 1)                       // force byte alignment

// Byte alignment matters for the structures contained within these pack pragmas.
// That's because it they goe over the USB cable and are used by the USB-I2C/IO's
// micro-controller.  The two processors (host and device) must therefore agree
// on exactly how things are aligned in memory, and the actual size in memory
// must be the same.


typedef struct _I2C_TRANS {
  BYTE byTransType;
  BYTE bySlvDevAddr;
  WORD wMemoryAddr;
  WORD wCount;
  BYTE Data[1088];
} I2C_TRANS, *PI2C_TRANS;

#pragma pack(pop)                          // restore previous alignment


WORD DAPI_GetDllVersion(void);
WORD DAPI_GetDriverVersion(void);
BOOL DAPI_GetFirmwareVersion(IN HANDLE hDevInstance, OUT LPWORD lpwVersion);

// open/close/info calls
HANDLE DAPI_OpenDeviceInstance(IN LPCSTR lpsDevName, IN BYTE byDevInstance);
BOOL DAPI_CloseDeviceInstance(HANDLE hDevInstance);
BOOL DAPI_DetectDevice(IN HANDLE hDevInstance);
BYTE DAPI_GetDeviceCount(IN LPCSTR lpsDevName);
BYTE DAPI_GetDeviceInfo(IN LPCSTR lpsDevName, OUT LPDEV_INFO_ARRAY lpDevInfoArray);
HANDLE DAPI_OpenDeviceBySerialId(IN LPCSTR lpsDevName, IN LPCSTR lpsDevSerialId);
BOOL DAPI_GetSerialId(IN HANDLE hDevInstance, OUT LPCSTR lpsDevSerialId);


// I/O calls
BOOL DAPI_ConfigIoPorts(IN HANDLE hDevInstance, IN ULONG ulIoPortConfig);
BOOL DAPI_GetIoConfig(IN HANDLE hDevInstance, OUT LPLONG lpulIoPortConfig);
BOOL DAPI_ReadIoPorts(IN HANDLE hDevInstance, OUT LPLONG lpulIoPortData);
BOOL DAPI_WriteIoPorts(IN HANDLE hDevInstance, IN ULONG ulIoPortData, IN ULONG ulIoPortMask);
LONG DAPI_BlockWriteIoPorts(IN HANDLE hDevInstance, IN WORD wBlockIoMode, IN WORD wBlockIoIndex, IN BYTE *pbyData, IN WORD wCount);
LONG DAPI_BlockReadIoPorts(IN HANDLE hDevInstance, OUT BYTE *pbyData, IN WORD wBlockIoMode, IN WORD wBlockIoIndex, IN WORD wCount);

// I2c calls
LONG DAPI_ReadI2c(IN HANDLE hDevInstance, OUT PI2C_TRANS TransI2C);
LONG DAPI_WriteI2c(IN HANDLE hDevInstance, IN PI2C_TRANS TransI2C);

// debugging calls
LONG DAPI_ReadDebugBuffer(LPCSTR lpsDebugString, IN HANDLE hDevInstance, IN LONG ulMaxBytes);

// fast transfer calls
LONG DAPI_WriteFastXferVr(IN HANDLE hDevInstance,
									WORD wCount, PBYTE pbyWriteData);
LONG DAPI_ReadFastXferVr(IN HANDLE hDevInstance,
									PBYTE pbyReadData, WORD wCount);
// generic USB transfer calls
BOOL DAPI_TransferData(IN HANDLE hDevInstance, BYTE byEndpoint, PBYTE pBuffer, PULONG pulLength);

BOOL DAPI_SetVendorRequest(IN HANDLE hDevInstance, IN BYTE byRequest, IN WORD wValue, IN WORD wIndex, IN WORD wLength, IN PBYTE pbySetData);

BOOL DAPI_GetVendorRequest(IN HANDLE hDevInstance, OUT PBYTE pbyGetData, IN BYTE byRequest, IN WORD wValue, IN WORD wIndex, IN WORD wLength);

// property calls
BOOL  DAPI_SetProperty(IN HANDLE hDevInstance, IN BYTE byPropIndex, IN BYTE byPropValue);
BOOL  DAPI_GetProperty(IN HANDLE hDevInstance, OUT LPBYTE lpbyPropValue, IN BYTE byPropIndex);

BOOL  DAPI_GetLastFirmwareError(IN HANDLE hDevInstance, OUT LPLONG lplError);

// unimplemented calls
void  DAPI_EnablePolling(void);
void  DAPI_DisablePolling(void);
void  DAPI_GetPolledInfo(void);

#if defined(__cplusplus)
}
#endif

// the following #endif is for preventing double includes of this header file
#endif